/*
 * Player.cpp
 *
 *  Created on: 29 Jan 2011
 *      Author: Ashley
 */

#include "Player.h"
#include "../DefaultGame.h"
#include "MyParticle.h"

Player::Player(b2World* world): 
	Object(),
	nozzleBounds(0,0,5),
	suckedItemTimers()
	{

	DefaultGame* dg = DefaultGame::getInstance(); 

	playerImage = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("player.png"))->scale(2.0f, 2.0f);
	playerImage->setCenterOfRotation(playerImage->getWidth()*0.5f, playerImage->getHeight()*0.5f);

	nozzleImage = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("player-tube.png"))->scale(2.0f, 2.0f);
	nozzleImage->setCenterOfRotation(nozzleImage->getWidth()*0.5f, nozzleImage->getHeight()*0.5f);


	// create a thing that moves -- player
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(w2b(160), w2b(200));
	dynamicBody = world->CreateBody(&bodyDef);

	b2PolygonShape dynamicBox;
	dynamicBox.SetAsBox(w2b(20), w2b(20));

	b2FixtureDef fixtureDef;
	fixtureDef.shape = &dynamicBox;
	fixtureDef.density = 100.0f;
	fixtureDef.friction = 0.3f;
	dynamicBody->CreateFixture(&fixtureDef);

	// vacuum tube sections.
	unsigned int numTubes = 12;
	float rightSize = 20.0f;
	b2Body* previousBody = dynamicBody;
	for(unsigned int i = 0; i < numTubes; i++) { 
		b2BodyDef bodyDef;
		bodyDef.type = b2_dynamicBody;
		bodyDef.position.Set(w2b(180 + (i*8)), w2b(200));
		b2Body* tubeBody = world->CreateBody(&bodyDef);

		b2PolygonShape dynamicBox;
		dynamicBox.SetAsBox(w2b(4), w2b(4));

		b2FixtureDef fixtureDef;
		fixtureDef.shape = &dynamicBox;
		fixtureDef.density = 100.0f;
		fixtureDef.friction = 0.3f;
		//fixtureDef.restitution = 0.2f;
		tubeBody->CreateFixture(&fixtureDef);

		b2RevoluteJointDef jointDef;
		jointDef.bodyA = previousBody;
		jointDef.bodyB = tubeBody;
		jointDef.localAnchorA.Set(w2b(rightSize), 0.0); 
		jointDef.localAnchorB.Set(0.0, 0.0); 
//		jointDef.maxLength = w2b(5);
		jointDef.collideConnected = false;
		b2RevoluteJoint* joint = (b2RevoluteJoint*) world->CreateJoint( &jointDef );

		previousBody = tubeBody;
		rightSize = 8.0f;

		nozzles.push_back(tubeBody);
		if (i == numTubes-1) {
			nozzle = tubeBody;
		}

	}
}   
   
void Player::reset() {  
	particleTimer = 0.0f;
}  

void Player::update(GameContainer* container, GameTimer* timer) {
	DefaultGame* dg = DefaultGame::getInstance();
	Input* in = ARK2D::getInput();

	if (in->isKeyDown(Input::MOUSE_BUTTON_LEFT)) {
		//nozzle->setLinearVelocity
		float dirx, diry = 0;
        float angle = MathUtil::anglef(nozzle->GetPosition().x*InGameState::worldScale, nozzle->GetPosition().y*InGameState::worldScale, in->getMouseX()+dg->stateInGame->cameraX, in->getMouseY());
		MathUtil::moveAngle<float>(dirx, diry, angle, 7.0f);
		nozzle->ApplyLinearImpulse( b2Vec2(dirx, diry), nozzle->GetWorldCenter(), true);
//        nozzle->ApplyForce(b2Vec2(dirx, diry), nozzle->GetWorldCenter(), true);

		//if (!dg->sfx_suck->isPlaying()) {
		//	dg->sfx_suck->play();
		//}

		particleTimer += timer->getDelta();
		if (particleTimer > 0.05f) { 
			particleTimer -= 0.05f;
			MyParticle* p = dg->stateInGame->particles->get();
			p->reset();
			p->type = 1;
			p->duration = MathUtil::randBetweenf(0.5f,0.75f);
			p->image = dg->stateInGame->particle;
			p->position.m_x = dg->stateInGame->player->nozzle->GetPosition().x*InGameState::worldScale, 
            p->position.m_y = dg->stateInGame->player->nozzle->GetPosition().y*InGameState::worldScale;
			// = in->getMouseX() + dg->stateInGame->cameraX;
			//p->position.m_y = in->getMouseY();
			MathUtil::moveAngle<float>(&p->position, angle, 40.0f);
			MathUtil::moveAngle<float>(&p->position, MathUtil::randBetweenf(0.0f, 360.0f), 15.0f);
			MathUtil::moveAngle<float>(&p->velocity, angle-180, 220.0f);
		}

	} else {  

		particleTimer = 0.05f;
		//if (dg->sfx_suck->isPlaying()) {
		//	dg->sfx_suck->stop();
		//}
	}


	b2Vec2 pos = nozzle->GetPosition();
	nozzleBounds.setLocationByCenter(pos.x * InGameState::worldScale, pos.y * InGameState::worldScale);

	for(unsigned int i = 0; i < suckedItemTimers.size(); i++) {
		suckedItemTimers[i] += timer->getDelta();
		if (suckedItemTimers[i] > 0.75f) {
			suckedItemTimers.erase(suckedItemTimers.begin() + i);
			break;
		}
	}
}
void Player::collision(Object* other) { 

} 

float Player::getX() {
	return dynamicBody->GetPosition().x * InGameState::worldScale;
}

void Player::render(GameContainer* container, Renderer* r) 
{
	b2Vec2 pos = dynamicBody->GetPosition();
	float angle = dynamicBody->GetAngle();
	//r->drawRect(pos.x, pos.y, 10, 10);
	playerImage->setDirty(true);
	playerImage->setRotation(MathUtil::toDegrees(angle));
	playerImage->drawCentered(pos.x*InGameState::worldScale, pos.y*InGameState::worldScale);

    float each = 0.75f / float(nozzles.size());
    signed int c = 0;
    float previousScale = 1.0f;
	for(signed int i = nozzles.size()-1; i >= 0; i--) {
		b2Vec2 pos = nozzles[c]->GetPosition();
		float angle = nozzles[c]->GetAngle();
		float nozzleScale = 1.0f;
		for(unsigned int j = 0; j < suckedItemTimers.size(); j++) {
			if (suckedItemTimers[j] > each*i && suckedItemTimers[j] < each*(i+1) ) {
				float diff = suckedItemTimers[j] - (each*i);
				nozzleScale = Easing::easebetween(Easing::QUADRATIC_IN, diff, 2.0f, 1.0f, each);
			}
		}
		float thisNozzleScale = nozzleScale;
		nozzleImage->setDirty(true);
		nozzleImage->setRotation(MathUtil::toDegrees(angle));
		nozzleImage->drawCenteredScaled(pos.x*InGameState::worldScale, pos.y*InGameState::worldScale, thisNozzleScale, thisNozzleScale);
		c++;
		previousScale = thisNozzleScale;
	}

	if (DefaultGame::getInstance()->stateInGame->debug) {
		nozzleBounds.render();
	}

}

Player::~Player() { 
	 
}
