/*
 * InGameState.cpp
 *
 *  Created on: 08 Jun 2011
 *      Author: Ashley
 */

#include "InGameState.h"
#include "../DefaultGame.h"

#include "../objects/Object.h"
#include "../objects/BlankObject.h"
#include "../objects/Player.h"
#include "../objects/RoomObject.h"
#include "../objects/ShelfObject.h"
#include "../objects/DecorationObject.h"
#include "../objects/MyParticle.h"
//#include "../objects/Pickup.h"
//#include "../objects/Explosion.h"
 
InGameState::InGameState(): 
	GameState(),  
    player(NULL),
    roomObjects(),
    shelfObjects() ,
    decorationObjects(),
    particles(NULL)
    //,
//	m_pickups(NULL),
//	m_explosions(NULL)
	{

}

unsigned int InGameState::id() {
	return StatesList::STATE_INGAME;
}

void InGameState::enter(GameContainer* container, StateBasedGame* game, GameState* from) { 

}
void InGameState::leave(GameContainer* container, StateBasedGame* game, GameState* to) { }



void InGameState::init(GameContainer* container, StateBasedGame* game) 
{
	
//	m_pickups = new Pool<Pickup*>(5);
//	m_explosions = new Pool<Explosion*>(5);

	DefaultGame* dg = DefaultGame::getInstance();
	backgroundImage = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("background-lines.png"))->scale(2.0f, 2.0f);
	
	debug = false;
	cameraX = 0;
	score = 0;

	levelTimerImageFull = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("timer-full.png"))->scale(2.0f, 2.0f);
	levelTimerImageEmpty = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("timer-empty.png"))->scale(2.0f, 2.0f);

	particles = new Pool<MyParticle*>(50);
	particle = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("particle.png"))->scale(2.0f, 2.0f); 
	reset();

	

	//start();
}


void InGameState::start() { 
    startLevel(0);
//	m_explosions->reset();
}
 
void InGameState::reset() {
    if (player != NULL) {
        player->reset();
    }
	particles->reset();

	score = 0;
	level = 0;
	levelTimer = 0.0f;
	levelDuration = 30.0f;
	levelObjectCount = 0;
	levelObjectDoneCount = 0;

	m_rotationEffectHappeningTimer = 0.0f;
	m_rotationEffectHappeningDuration = 1.0f;
	m_rotationEffectHalfWayEvent = false;

//	m_pickups->reset();
}
void InGameState::startLevel(unsigned int l) {
    DefaultGame* dg = DefaultGame::getInstance();
	level = l;

	decorationObjects.clear(); 	// memory leak
	shelfObjects.clear(); 		// memory leak
	roomObjects.clear(); 		// memory leak

	levelObjectCount = 0;
	levelObjectDoneCount = 0;

	int worldWidthMax = 480 + (240*level);	

	b2Vec2 gravity(0.0f, 10.0f);
	world = new b2World(gravity);
	worldWidth = MathUtil::randBetweenf(480, worldWidthMax);

	// create floor
	b2BodyDef groundBodyDef;
	groundBodyDef.position.Set(w2b(worldWidth/2), w2b(296));

	groundBody = world->CreateBody(&groundBodyDef);
	
	b2PolygonShape groundBox;
	groundBox.SetAsBox(w2b(worldWidth/2), w2b(24));
	groundBody->CreateFixture(&groundBox, 0.0f);
	{
		b2BodyDef groundBodyDef;
		groundBodyDef.position.Set(w2b(worldWidth/2), w2b(24));

		b2Body* body = world->CreateBody(&groundBodyDef);
		
		b2PolygonShape groundBox;
		groundBox.SetAsBox(w2b(worldWidth/2), w2b(24));
		body->CreateFixture(&groundBox, 0.0f);
	}
	{
		// create left wall
		b2BodyDef groundBodyDef;
		groundBodyDef.position.Set(w2b(20), w2b(160));

		b2Body* body = world->CreateBody(&groundBodyDef);
		
		b2PolygonShape groundBox;
		groundBox.SetAsBox(w2b(1), w2b(160));
		body->CreateFixture(&groundBox, 0.0f);
	}
	{
		// create right wall
		b2BodyDef groundBodyDef;
		groundBodyDef.position.Set(w2b(worldWidth-20), w2b(160));

		b2Body* body = world->CreateBody(&groundBodyDef);
		
		b2PolygonShape groundBox;
		groundBox.SetAsBox(w2b(1), w2b(160));
		body->CreateFixture(&groundBox, 0.0f);
	}

	player = new Player(world);

	Image* books1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-books-flat.png"))->scale(2.0f, 2.0f);
	books1->setCenterOfRotation(books1->getWidth()*0.5f, books1->getHeight()*0.5f);
	
	Image* books2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-books-shelf.png"))->scale(2.0f, 2.0f);
	books2->setCenterOfRotation(books2->getWidth()*0.5f, books2->getHeight()*0.5f);

	Image* fire1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-fireextinguisher-1.png"))->scale(2.0f, 2.0f);
	fire1->setCenterOfRotation(fire1->getWidth()*0.5f, fire1->getHeight()*0.5f);
	
	Image* fire2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-fireextinguisher-2.png"))->scale(2.0f, 2.0f);
	fire2->setCenterOfRotation(fire2->getWidth()*0.5f, fire2->getHeight()*0.5f);

	Image* jar = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-jar.png"))->scale(2.0f, 2.0f);
	jar->setCenterOfRotation(jar->getWidth()*0.5f, jar->getHeight()*0.5f);
	
	Image* junk1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-junk-1.png"))->scale(2.0f, 2.0f);
	junk1->setCenterOfRotation(junk1->getWidth()*0.5f, junk1->getHeight()*0.5f);
	
	Image* junk2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-junk-2.png"))->scale(2.0f, 2.0f);
	junk2->setCenterOfRotation(junk2->getWidth()*0.5f, junk2->getHeight()*0.5f);

	Image* junk3 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-junk-3.png"))->scale(2.0f, 2.0f);
	junk3->setCenterOfRotation(junk3->getWidth()*0.5f, junk3->getHeight()*0.5f);	

	Image* junk4 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-junk-4.png"))->scale(2.0f, 2.0f);
	junk4->setCenterOfRotation(junk4->getWidth()*0.5f, junk4->getHeight()*0.5f);

	Image* lamp = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-lamp.png"))->scale(2.0f, 2.0f);
	lamp->setCenterOfRotation(lamp->getWidth()*0.5f, lamp->getHeight()*0.5f);

	Image* lightbulb = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-lightbulb.png"))->scale(2.0f, 2.0f);
	lightbulb->setCenterOfRotation(lightbulb->getWidth()*0.5f, lightbulb->getHeight()*0.5f);
	
	Image* roomba = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-roomba.png"))->scale(2.0f, 2.0f);
	roomba->setCenterOfRotation(roomba->getWidth()*0.5f, roomba->getHeight()*0.5f);
	
	Image* tshirt = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-tshirt.png"))->scale(2.0f, 2.0f);
	tshirt->setCenterOfRotation(tshirt->getWidth()*0.5f, tshirt->getHeight()*0.5f);
	
	Image* vase1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-vase-1.png"))->scale(2.0f, 2.0f);
	vase1->setCenterOfRotation(vase1->getWidth()*0.5f, vase1->getHeight()*0.5f);

	Image* vase2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-vase-2.png"))->scale(2.0f, 2.0f);
	vase2->setCenterOfRotation(vase2->getWidth()*0.5f, vase2->getHeight()*0.5f);
	
	Image* wheel = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-wheel.png"))->scale(2.0f, 2.0f);
	wheel->setCenterOfRotation(wheel->getWidth()*0.5f, wheel->getHeight()*0.5f);
	
	Image* basket = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-basket.png"))->scale(2.0f, 2.0f);
	basket->setCenterOfRotation(basket->getWidth()*0.5f, basket->getHeight()*0.5f);

	Image* card = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-card.png"))->scale(2.0f, 2.0f);
	card->setCenterOfRotation(card->getWidth()*0.5f, card->getHeight()*0.5f);
	
	Image* apple = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-apple.png"))->scale(2.0f, 2.0f);
	apple->setCenterOfRotation(apple->getWidth()*0.5f, apple->getHeight()*0.5f);

	Image* banana = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-banana.png"))->scale(2.0f, 2.0f);
	banana->setCenterOfRotation(banana->getWidth()*0.5f, banana->getHeight()*0.5f);

	Image* fruitbowl = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-fruitbowl.png"))->scale(2.0f, 2.0f);
	fruitbowl->setCenterOfRotation(fruitbowl->getWidth()*0.5f, fruitbowl->getHeight()*0.5f);

	Image* jug = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-jug.png"))->scale(2.0f, 2.0f);
	jug->setCenterOfRotation(jug->getWidth()*0.5f, jug->getHeight()*0.5f);	

	Image* kettle = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-kettle.png"))->scale(2.0f, 2.0f);
	kettle->setCenterOfRotation(kettle->getWidth()*0.5f, kettle->getHeight()*0.5f);

	Image* liquid = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-liquid.png"))->scale(2.0f, 2.0f);
	liquid->setCenterOfRotation(liquid->getWidth()*0.5f, liquid->getHeight()*0.5f);

	Image* microwave = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-microwave.png"))->scale(2.0f, 2.0f);
	microwave->setCenterOfRotation(microwave->getWidth()*0.5f, microwave->getHeight()*0.5f);

	Image* wineglass = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-wineglass.png"))->scale(2.0f, 2.0f);
	wineglass->setCenterOfRotation(wineglass->getWidth()*0.5f, wineglass->getHeight()*0.5f);

	Image* toiletroll = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-toiletroll.png"))->scale(2.0f, 2.0f);
	toiletroll->setCenterOfRotation(toiletroll->getWidth()*0.5f, toiletroll->getHeight()*0.5f);

	Image* toiletbrush = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-toiletbrush.png"))->scale(2.0f, 2.0f);
	toiletbrush->setCenterOfRotation(toiletbrush->getWidth()*0.5f, toiletbrush->getHeight()*0.5f);

	Image* towel1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-towel-1.png"))->scale(2.0f, 2.0f);
	towel1->setCenterOfRotation(towel1->getWidth()*0.5f, towel1->getHeight()*0.5f);

	Image* towel2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-towel-2.png"))->scale(2.0f, 2.0f);
	towel2->setCenterOfRotation(towel2->getWidth()*0.5f, towel2->getHeight()*0.5f);

	Image* cup = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-cup.png"))->scale(2.0f, 2.0f);
	cup->setCenterOfRotation(cup->getWidth()*0.5f, cup->getHeight()*0.5f);

	Image* toothbrush = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-toothbrush.png"))->scale(2.0f, 2.0f);
	toothbrush->setCenterOfRotation(toothbrush->getWidth()*0.5f, toothbrush->getHeight()*0.5f);

	Image* tube = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-tube.png"))->scale(2.0f, 2.0f);
	tube->setCenterOfRotation(tube->getWidth()*0.5f, tube->getHeight()*0.5f);

	Image* terry = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-terry.png"))->scale(2.0f, 2.0f);
	terry->setCenterOfRotation(terry->getWidth()*0.5f, terry->getHeight()*0.5f);

	Image* teacup = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-teacup.png"))->scale(2.0f, 2.0f);
	teacup->setCenterOfRotation(teacup->getWidth()*0.5f, teacup->getHeight()*0.5f);

	
	vector<Image*> randomObjects;
	vector<Image*> randomShelfObjects;
	vector<Image*> randomPaintingObjects;
	
	vector<Image*> randomLoungeObjects;
	vector<Image*> randomBathroomObjects;
	vector<Image*> randomKitchenObjects;

	// all levels
	randomObjects.push_back(junk1);
	randomObjects.push_back(junk2);
	randomObjects.push_back(junk3);
	randomObjects.push_back(junk4);
	randomObjects.push_back(lightbulb);
	randomObjects.push_back(jar);

	// lounge
	randomLoungeObjects.push_back(card);
	randomLoungeObjects.push_back(wheel);
	//randomLoungeObjects.push_back(vase2);
	//randomLoungeObjects.push_back(vase1);
	randomLoungeObjects.push_back(roomba);
	randomLoungeObjects.push_back(books1);
	randomLoungeObjects.push_back(books2);
	randomLoungeObjects.push_back(lamp);
	randomLoungeObjects.push_back(tshirt);

	// bathroom
	randomBathroomObjects.push_back(basket);
	randomBathroomObjects.push_back(roomba);
	randomBathroomObjects.push_back(tshirt);
	randomBathroomObjects.push_back(toiletbrush);
	randomBathroomObjects.push_back(toiletroll);
	randomBathroomObjects.push_back(toothbrush);
	randomBathroomObjects.push_back(tube);
	randomBathroomObjects.push_back(towel1);
	randomBathroomObjects.push_back(towel2);

	// shelves (all levels)
	randomShelfObjects.push_back(card);
	randomShelfObjects.push_back(vase1);
	randomShelfObjects.push_back(vase2);
	randomShelfObjects.push_back(books1);
	randomShelfObjects.push_back(books2);
	randomShelfObjects.push_back(teacup);

	// kitchen 
	randomKitchenObjects.push_back(banana);
	randomKitchenObjects.push_back(fruitbowl);
	randomKitchenObjects.push_back(jug);
	randomKitchenObjects.push_back(kettle);
	//randomKitchenObjects.push_back(liquid);
	randomKitchenObjects.push_back(microwave);
	randomKitchenObjects.push_back(wineglass);
	randomKitchenObjects.push_back(basket);
	randomKitchenObjects.push_back(jar);
	randomKitchenObjects.push_back(fire1);
	randomKitchenObjects.push_back(fire2);
	randomKitchenObjects.push_back(terry);
	

	

	// decorations
	Image* door = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-door.png"))->scale(2.0f, 2.0f);
	door->setCenterOfRotation(door->getWidth()*0.5f, door->getHeight()*0.5f);

	Image* sofa = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-sofa.png"))->scale(2.0f, 2.0f);
	sofa->setCenterOfRotation(sofa->getWidth()*0.5f, sofa->getHeight()*0.5f);

	Image* window = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-window.png"))->scale(2.0f, 2.0f);
	window->setCenterOfRotation(window->getWidth()*0.5f, window->getHeight()*0.5f);

	Image* toilet = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-toilet.png"))->scale(2.0f, 2.0f);
	toilet->setCenterOfRotation(toilet->getWidth()*0.5f, toilet->getHeight()*0.5f);

	Image* bathtub = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-bathtub.png"))->scale(2.0f, 2.0f);
	bathtub->setCenterOfRotation(bathtub->getWidth()*0.5f, bathtub->getHeight()*0.5f);

    Image* sink = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-sink.png"))->scale(2.0f, 2.0f);
    sink->setCenterOfRotation(sink->getWidth()*0.5f, sink->getHeight()*0.5f);
    
	Image* oven = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-oven.png"))->scale(2.0f, 2.0f);
	oven->setCenterOfRotation(oven->getWidth()*0.5f, oven->getHeight()*0.5f);

	Image* washingmachine = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-washingmachine.png"))->scale(2.0f, 2.0f);
	washingmachine->setCenterOfRotation(washingmachine->getWidth()*0.5f, washingmachine->getHeight()*0.5f);

	// paintings
	Image* painting1 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-painting-1.png"))->scale(2.0f, 2.0f);
	painting1->setCenterOfRotation(painting1->getWidth()*0.5f, 0.0f);
	randomPaintingObjects.push_back(painting1);

	Image* painting2 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-painting-2.png"))->scale(2.0f, 2.0f);
	painting2->setCenterOfRotation(painting2->getWidth()*0.5f, 0.0f);
	randomPaintingObjects.push_back(painting2);

	Image* painting3 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-painting-3.png"))->scale(2.0f, 2.0f);
	painting3->setCenterOfRotation(painting3->getWidth()*0.5f, 0.0f);
	randomPaintingObjects.push_back(painting3);

	Image* painting4 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-painting-4.png"))->scale(2.0f, 2.0f);
	painting4->setCenterOfRotation(painting4->getWidth()*0.5f, 0.0f);
	randomPaintingObjects.push_back(painting4);

	Image* painting5 = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("object-painting-5.png"))->scale(2.0f, 2.0f);
	painting5->setCenterOfRotation(painting5->getWidth()*0.5f, 0.0f);
	randomPaintingObjects.push_back(painting5);


	// proc gen shelves
	Image* shelf = (Image*) dg->sheet->getSubImage(dg->desc->getItemByName("decoration-shelf.png"))->scale(2.0f, 2.0f);
	unsigned int numShelves = floor(worldWidth/360);
	for (unsigned int i = 0; i < numShelves; i++) {
		ShelfObject* so = new ShelfObject();
		so->image = shelf;
		float shelfX = (i*360) + 200 + MathUtil::randBetweenf(0, 80);
		float shelfY = MathUtil::randBetweenf(100, 200);
		so->set(80, shelfX, shelfY);
		shelfObjects.push_back(so);

		// add a bunch of random stuff to this shelf.
		int w = 0;
		while(w < 80) {
			Image* item = randomShelfObjects.at(MathUtil::randBetween(0, randomShelfObjects.size()));
			if (w + item->getWidth() > 80) {
				break;
			}

			RoomObject* ro = new RoomObject();
			ro->set(item, shelfX - 40 + w, shelfY - (item->getHeight()*0.5f));
			roomObjects.push_back(ro);
			levelObjectCount++;

            w += item->getWidth();

		}

		Image* painting = randomPaintingObjects[MathUtil::randBetween(0, randomPaintingObjects.size())];
		DecorationObject* doorObject = new DecorationObject();
    	doorObject->set(painting, shelfX - 40 + w + MathUtil::randBetweenf(120, 240),120,0.5f,1.0f);
		decorationObjects.push_back(doorObject);
	}

	vector<Image*> thisObjects;
	for(unsigned int i = 0; i < randomObjects.size(); i++) {
		thisObjects.push_back(randomObjects[i]);
	}
	if (level % 3 == 0) {
		for(unsigned int i = 0; i < randomLoungeObjects.size(); i++) {
			thisObjects.push_back(randomLoungeObjects[i]);
		}	
	} else if (level % 3 == 1) {
		for(unsigned int i = 0; i < randomKitchenObjects.size(); i++) {
			thisObjects.push_back(randomKitchenObjects[i]);
		}	
	} else if (level % 3 == 2) {
		for(unsigned int i = 0; i < randomBathroomObjects.size(); i++) {
			thisObjects.push_back(randomBathroomObjects[i]);
		}	
	}

	// add a bunch of random stuff to the floor...
	unsigned int items = (level+1)*8;
	for(unsigned int i = 0; i < items; i++) { 
		Image* img = thisObjects[MathUtil::randBetween(0, thisObjects.size())];
		RoomObject* ro = new RoomObject();
		ro->set(img, MathUtil::randBetweenf(50.0f, worldWidth-50.0f), 272 - (img->getHeight()*0.5f));
		roomObjects.push_back(ro);
		levelObjectCount++;
	}
	

	



	DecorationObject* doorObject = new DecorationObject();
    doorObject->set(door,worldWidth-100,272,0.5f,1.0f);
	decorationObjects.push_back(doorObject);

	// paintings
	
	// window
	DecorationObject* windowObject = new DecorationObject();
    windowObject->set(window,100,140,0.5f,0.5f);
	decorationObjects.push_back(windowObject);

	// sofa
	if (level % 3 == 0) { 
		DecorationObject* sofaObject = new DecorationObject();
	    sofaObject->set(sofa,MathUtil::randBetweenf(200, worldWidth - 400),272,0.5f,1.0f);
		decorationObjects.push_back(sofaObject);
	} else if (level % 3 == 1) { 
		float ovx = MathUtil::randBetweenf(50, 200);
		DecorationObject* ovenObject = new DecorationObject();
	    ovenObject->set(oven,ovx,272,0.5f,1.0f);
		decorationObjects.push_back(ovenObject);

		DecorationObject* washingMachineObject = new DecorationObject();
	    washingMachineObject->set(washingmachine,MathUtil::randBetweenf(ovx+100, worldWidth - 200),272,0.5f,1.0f);
		decorationObjects.push_back(washingMachineObject);
	} else if (level % 3 == 2) {
		DecorationObject* toiletObject = new DecorationObject();
	    toiletObject->set(toilet,(worldWidth/100.0f)*20,272,0.5f,1.0f);
		decorationObjects.push_back(toiletObject);	

		DecorationObject* sinkObject = new DecorationObject();
	    sinkObject->set(sink,(worldWidth/100.0f)*30,272,0.5f,1.0f);
		decorationObjects.push_back(sinkObject);	

		DecorationObject* bathtubObject = new DecorationObject();
	    bathtubObject->set(bathtub,(worldWidth/100.0f)*60,272,0.5f,1.0f);
		decorationObjects.push_back(bathtubObject);	
	}
	
}

void InGameState::update(GameContainer* container, StateBasedGame* game, GameTimer* timer) {
	DefaultGame* dg = DefaultGame::getInstance();
	Input* in = ARK2D::getInput();

	if (in->isKeyPressed(Input::KEY_ESCAPE) || in->isKeyPressed(Input::KEY_ENTER) || in->isGamepadButtonPressed(Gamepad::BUTTON_START)) {
	//if (i->isKeyPressed(Input::KEY_ENTER) ||  || i->isGamepadButtonPressed(Gamepad::BUTTON_A)) {
		//dg->sfx_ui_select->play();
		game->enterState(dg->stateMenu);
	}

	levelTimer += timer->getDelta();
	if (levelTimer >= levelDuration) {
		//fail();

		dg->sfx_die->play();
		dg->enterState(dg->stateSplash);

		levelTimer = levelDuration;
	}
	
 	cameraX = 0;
 	if (player->getX() >= worldWidth - 240) {
 		cameraX = worldWidth - 480;
 	} else if (player->getX() >= 240) {
 		cameraX = player->getX() - 240;
 	} else {
 		cameraX = 0;
 	}
	
	if (in->isKeyPressed(Input::KEY_D)) {
		debug = !debug;
	}
	
	particles->updateAll(container, timer);
	

	float32 timeStep = 1.0f / 60.0f;
	int32 velocityIterations = 6;   
	int32 positionIterations = 2;
	world->Step(timeStep, velocityIterations, positionIterations);
	
	player->update(container, timer);

	for(unsigned int i = 0; i < roomObjects.size(); i++) {
    	roomObjects[i]->update();
    }
	
	// check nozzle with objects.
	for(unsigned int i = 0; i < roomObjects.size(); ++i) { 
		bool b = Shape<float>::collision_circleCircle(
			player->nozzleBounds.getCenterX(),
			player->nozzleBounds.getCenterY(),
			player->nozzleBounds.getRadius(),
			roomObjects[i]->dynamicBody->GetPosition().x*InGameState::worldScale,
			roomObjects[i]->dynamicBody->GetPosition().y*InGameState::worldScale,
			roomObjects[i]->radius
		);
		if (roomObjects[i]->scaleTimer == 0.0f && b) {
			roomObjects[i]->scaleTimer = 0.01f;
			dg->sfx_items[MathUtil::randBetween(0, dg->sfx_items.size())]->play();

			player->suckedItemTimers.push_back(0.01f);
			score++;
		}

		if (roomObjects[i]->scaleTimer > 0.0f) {
			roomObjects[i]->scaleTimer += timer->getDelta();
			if (roomObjects[i]->scaleTimer >= roomObjects[i]->scaleDuration) {
				
				world->DestroyBody(roomObjects[i]->dynamicBody);
				roomObjects[i]->dynamicBody = NULL;	

				delete roomObjects[i];
				roomObjects[i] = NULL;

				levelObjectDoneCount++;
				roomObjects.erase(roomObjects.begin() + i);
				i = 0;
				continue;
			}
 		}
	}

	

	if (levelObjectDoneCount == levelObjectCount && m_rotationEffectHappeningTimer == 0.0f) { // && m_waveTimer >= m_waveDuration) {
		m_rotationEffectHappeningTimer = 0.01f;	
		DefaultGame::getInstance()->sfx_levelUp->play();
	}

	if (m_rotationEffectHappeningTimer > 0.0f) {
		levelTimer = 0.0f;
		m_rotationEffectHappeningTimer += timer->getDelta();
		if (m_rotationEffectHappeningTimer >= m_rotationEffectHappeningDuration*0.5f && !m_rotationEffectHalfWayEvent) {
			startLevel(level + 1);
			m_rotationEffectHalfWayEvent = true; 
			score += 5;
		}
		if (m_rotationEffectHappeningTimer >= m_rotationEffectHappeningDuration) {
			m_rotationEffectHappeningTimer = 0.0f;
			m_rotationEffectHalfWayEvent = false;
		}
	}		


}

void InGameState::render(GameContainer* container, StateBasedGame* game, Renderer* r) {
	//r->drawString("InGame State", 20, 20, Renderer::ALIGN_LEFT, Renderer::ALIGN_TOP);

	// Boilerplate
	//r->getBatch()->setEnabled(true);
	//
	//r->getBatch()->render(); 
	//r->getBatch()->setEnabled(false);

	Color backgroundColor;
	if (level % 3 == 0) { 
		backgroundColor = Color("#222a3f"); 
	} else if (level % 3 == 1) { 
		backgroundColor = Color("#1c3333"); 
	}else if (level % 3 == 2) { 
		backgroundColor = Color("#3b2025"); 
	}
	container->setClearColor(backgroundColor);
	r->setScissorBoxColors(backgroundColor,backgroundColor,backgroundColor,backgroundColor);

	

	backgroundImage->draw(28, 44);

	string pcc = Cast::toString<unsigned int>(ceil((float(levelObjectDoneCount) / float(levelObjectCount))*100));
	r->drawString(StringUtil::append("SCORE: ", StringUtil::zeropad(score, 5)), 42, 31, Renderer::ALIGN_LEFT, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
    r->drawString(StringUtil::append(pcc,"%"), 480-42, 31, Renderer::ALIGN_RIGHT, Renderer::ALIGN_CENTER, 0.0f, 2.0f);
    r->drawString("TIME", 240, 320-20, Renderer::ALIGN_CENTER, Renderer::ALIGN_CENTER, 0.0f, 2.0f);

    // timer
    float each = levelDuration / 29.0f;
    for(unsigned int i = 0; i < 29; i++) {
    	
    	float lt = levelDuration - levelTimer;
    	if (lt < (i+1) * each) {
			levelTimerImageEmpty->draw(38 + (14*i), 280); 	
    	} else {
    		levelTimerImageFull->draw(38 + (14*i), 280);     		
    	}
    }


    r->pushMatrix();
    r->translate(240.0f, 160.0f);
    m_rotationEffectAlpha = 1.0f;
	if (m_rotationEffectHappeningTimer > 0.0f) {
		

		float rot = Easing::easebetween(Easing::QUADRATIC_IN_OUT, m_rotationEffectHappeningTimer, 0.0f, 360.0f, m_rotationEffectHappeningDuration);
		r->rotate(rot);	

		float scaleAmount = 2.0f;
		float sc = Easing::easebetween(Easing::QUADRATIC_IN_OUT, m_rotationEffectHappeningTimer, 1.0f, scaleAmount, m_rotationEffectHappeningDuration/2);		
		if (m_rotationEffectHappeningTimer >= m_rotationEffectHappeningDuration*0.5) {
			sc = Easing::easebetween(Easing::QUADRATIC_IN_OUT, m_rotationEffectHappeningTimer - (m_rotationEffectHappeningDuration*0.5), scaleAmount, 1.0f, m_rotationEffectHappeningDuration/2);
		}
		r->scale(sc, sc);


		m_rotationEffectAlpha = Easing::easebetween(Easing::QUADRATIC_OUT, m_rotationEffectHappeningTimer, 1.0f, 0.0f, m_rotationEffectHappeningDuration/2);
		if (m_rotationEffectHappeningTimer >= m_rotationEffectHappeningDuration*0.5) {
			m_rotationEffectAlpha = Easing::easebetween(Easing::QUADRATIC_IN, m_rotationEffectHappeningTimer - (m_rotationEffectHappeningDuration*0.5), 0.0f, 1.0f, m_rotationEffectHappeningDuration/2);
		}

		if (m_rotationEffectHappeningTimer >= m_rotationEffectHappeningDuration*0.5) { 
			float newTeleportX = Easing::easebetween(Easing::QUADRATIC_IN_OUT, m_rotationEffectHappeningTimer - (m_rotationEffectHappeningDuration*0.5), m_rotationEffectTranslatePointStartX, m_rotationEffectTranslatePointX, m_rotationEffectHappeningDuration/2);
			float newTeleportY = Easing::easebetween(Easing::QUADRATIC_IN_OUT, m_rotationEffectHappeningTimer - (m_rotationEffectHappeningDuration*0.5), m_rotationEffectTranslatePointStartY, m_rotationEffectTranslatePointY, m_rotationEffectHappeningDuration/2);
//			m_teleportSprite->position.set(newTeleportX, newTeleportY, 0);
		}
	}
	r->translate(-240.0f, -160.0f);


	r->pushMatrix();
	r->translate(cameraX*-1, 0.0f);

	r->setDrawColorf(1.0f, 1.0f, 1.0f, m_rotationEffectAlpha);
    player->render(container, r);

    for(unsigned int i = 0; i < decorationObjects.size(); i++) {
    	decorationObjects[i]->render();
    }
    for(unsigned int i = 0; i < shelfObjects.size(); i++) {
    	shelfObjects[i]->render();
    }
    for(unsigned int i = 0; i < roomObjects.size(); i++) {
    	roomObjects[i]->render();
    }

	particles->renderAll(container, r);
   

	if (debug) { 

		b2Body* body = world->GetBodyList();
		while (body != NULL) {
			b2Vec2 pos = body->GetPosition();
			float angle = body->GetAngle();
			//r->drawRect(pos.x, pos.y, 4, 4);

			b2Fixture* fix = body->GetFixtureList();
			while (fix != NULL) { 
				b2Shape* f = (b2Shape*) fix->GetShape();
				switch (f->GetType()) {
					case b2Shape::e_polygon: 
					{
						b2PolygonShape* ps = (b2PolygonShape*) f;
						int vertexc = ps->GetVertexCount();
						for(unsigned int i = 0; i < vertexc; i++) {
							const b2Vec2& v1 = ps->GetVertex(i);
							const b2Vec2& v2 = ps->GetVertex((i+1) % vertexc);
							r->drawLine((pos.x + v1.x)*worldScale, (pos.y + v1.y)*worldScale, (pos.x + v2.x)*worldScale, (pos.y + v2.y)*worldScale);
						}
					}
	                case b2Shape::e_circle:
	                    break;
	                case b2Shape::e_chain:
	                    break;
	                case b2Shape::e_edge:
	                    break;
	                case b2Shape::e_typeCount:
	                    break;
				}
				
				fix = fix->GetNext();
			}
			body = body->GetNext();
		}
		

		b2Joint* joint = world->GetJointList();
		while (joint != NULL) {
			const b2Vec2& a = joint->GetAnchorA();
			const b2Vec2& b = joint->GetAnchorB();
			//r->setDrawColor(Color::cyan);
			r->drawLine(a.x * worldScale, a.y * worldScale, b.x * worldScale, b.y * worldScale);
			joint = joint->GetNext();
		}
	}

	r->popMatrix();
	r->popMatrix();
}

InGameState::~InGameState() {

}

